@@ -8,13 +8,13 @@
 
 (defun machine-type () "Acorn")
 (defun machine-version () "Risc PC, OS 3.7")
-(defun machine-instance () (or (sys::getenv "HOSTNAME") "edit config.lisp"))
+(defun machine-instance () (or (getenv "HOSTNAME") "edit config.lisp"))
 
-(defun short-site-name () (or (sys::getenv "ORGANIZATION") "edit config.lisp"))
-(defun long-site-name () (or (sys::getenv "ORGANIZATION") "edit config.lisp"))
+(defun short-site-name () (or (getenv "ORGANIZATION") "edit config.lisp"))
+(defun long-site-name () (or (getenv "ORGANIZATION") "edit config.lisp"))
 
 (defparameter *editor* "filer_run" "The name of the editor.")
-(defun editor-name () (or (sys::getenv "EDITOR") *editor*))
+(defun editor-name () (or (getenv "EDITOR") *editor*))
 
 (defun edit-file (file)
   "(edit-file file) edits a file."
@@ -24,21 +24,17 @@
       (format nil "~A ~A"
                   (editor-name)
                   (if (pathname-type pathname)
-                    ; swap pathname's name and type
+                    ;; swap pathname's name and type
                     (merge-pathnames
                       (make-pathname :name (pathname-type pathname)
-                                     :type (pathname-name pathname)
-                      )
-                      pathname
-                    )
-                    pathname
-                  )
-) ) ) )
+                                     :type (pathname-name pathname))
+                      pathname)
+                    pathname)))))
 
 (defun editor-tempfile ()
   "The temporary file LISP creates for editing."
   ;; We write this instead of "<Wimp$ScrapDir>.lisptemp" in order to
-  ;; make sure that all the components of (sys::getenv "Wimp$ScrapDir")
+  ;; make sure that all the components of (getenv "Wimp$ScrapDir")
   ;; are treated as directory components.
   (merge-pathnames "lisptemp" "<Wimp$ScrapDir>."))
 
@@ -48,11 +44,14 @@
 ;; This makes screen output prettier:
 (setq *print-pretty* t)
 
+;; which browser do you use? (see `*browsers*' in clhs.lisp)
+;; (setq *browser* :mozilla-remote)
+
 ;; Common Lisp HyperSpec access
 (defvar *clhs-root-default*)
 (defun clhs-root ()
   "This returns the root URL for the Common Lisp HyperSpec.
 You can set the environment variable `CLHSROOT' or redefine this function
 in ~/.clisprc.  On win32 you can also use the Registry."
-  (or (sys::getenv "CLHSROOT") *clhs-root-default*))
+  (or (getenv "CLHSROOT") *clhs-root-default*))
 (setq *clhs-root-default* "http://www.lisp.org/HyperSpec/")
