@@ -4,21 +4,31 @@
 
 (in-package "EXT")
 
-(export '(clhs clhs-root *browsers* *browser* read-from-file browse-url))
+(export '(clhs clhs-root read-from-file browse-url))
 
 (in-package "SYSTEM")
 
 (defvar *clhs-table* nil)       ; the hash table
 
-(defvar *browsers*              ; alist of browsers & commands
-  '((:netscape "netscape" "-remote" "openURL(~a,new-window)")
+(defvar *browsers*
+  '((:netscape "netscape" "~a")
+    (:netscape-remote "netscape" "-remote" "openURL(~a,new-window)")
+    (:mozilla "mozilla" "~a")
+    (:mozilla-remote "mozilla" "-remote" "openURL(~a,new-window)")
     (:konqueror "kfmclient" "openURL" "~a")
-    (:lynx "xterm" "-e" "lynx" "~a")
-    (:w3m "xterm" "-e" "w3m" "~a")
+    (:lynx "lynx" "~a")
+    #+unix (:lynx-xterm "xterm" "-e" "lynx" "~a")
+    (:links "links" "~a")
+    #+unix (:links-xterm "xterm" "-e" "links" "~a")
+    (:w3m "w3m" "~a")
+    #+unix (:w3m-xterm "xterm" "-e" "w3m" "~a")
     (:mmm "mmm" "-external" "~a")
     (:mosaic "xmosaic" "~a")
-    (:emacs-w3 "gnudoit" "-q" "(w3-fetch \"~a\")")))
-(defvar *browser* nil)          ; the default browser
+    (:emacs-w3 "gnudoit" "-q" "(w3-fetch \"~a\")"))
+  "Alist of browsers and commands that invoke them.
+`~a' will be replaced with the URL to view.")
+(defvar *browser* nil
+  "The default browser - a key in `*browsers*' or a list of strings.")
 
 (defun read-from-file (file &key (out *standard-output*)
                        (package (find-package "KEYWORD")))
@@ -44,6 +54,14 @@ The keyword argument OUT specifies the output for log messages."
 
 (defun browse-url (url &key (browser *browser*) (out *standard-output*))
   "Run the browser (a keyword in `*browsers*' or a list) on the URL."
+  #+WIN32
+  (unless browser ;feed url to ShellExecute if no browser is specified
+    (when out
+      (format out "~&;; starting the default system browser with url ~s..." url)
+      (force-output (if (eq out t) *standard-output* out)))
+    (ext::shell-execute "open" url nil nil) ;to start default browser
+    (when out (format out "done~%"))
+    (return-from browse-url))
   (let* ((command
           (etypecase browser
             (list browser)
@@ -55,7 +73,7 @@ The keyword argument OUT specifies the output for log messages."
            (when out
              (format out "~&;; running [~s~{ ~s~}]..." (car command) args)
              (force-output (if (eq out t) *standard-output* out)))
-           (run-program (car command) :arguments args :wait nil)
+	   (#+WIN32 ext::launch #-WIN32 run-program (car command) :arguments args :wait nil)
            (when out
              (format out "done~%")))
           ((format t "~s: no browser specified; please point your browser at
