@@ -4,6 +4,7 @@
 (export '(custom::*prompt*) "CUSTOM")
 (export
  '(*command-index* prompt-new-package package-short-name
+   #+unix make-xterm-io-stream
    custom::*prompt*)
  "EXT")
 (in-package "SYSTEM")
@@ -61,7 +62,7 @@
                         (prompt-new-package))
                   (package-short-name *package*))
                 (incf *command-index*))
-        (ENGLISH "[*package* invalid]")))
+        (TEXT "[*package* invalid]")))
   "The top level prompt.  If a function, the return value is used.
 If anything else, printed.")
 
@@ -78,17 +79,18 @@ If anything else, printed.")
 (defun prompt-string3 () "> ")
 
 ;; Help-function:
-(defvar *key-bindings* nil)             ; List of Key-Bindings  und Helpstrings
+(defvar *key-bindings* nil)     ; list of key-bindings and help strings
 (defun help ()
-  (dolist (s (reverse (remove-if-not #'stringp *key-bindings*)))
-    (write-string s #|*debug-io*|#)))
+  (dolist (s *key-bindings*)
+    (when (stringp s)
+      (write-string s #|*debug-io*|#))))
 
 (defvar *saved-debug-package* *common-lisp-user-package*)
 (defvar *saved-debug-readtable* (copy-readtable nil))
 (defun debug-reset-io ()
   (rotatef *package* *saved-debug-package*)
   (rotatef *readtable* *saved-debug-readtable*)
-  (format *debug-io* "~&Reset *PACKAGE* to ~s" *package*)
+  (format *debug-io* (TEXT "~&Reset *PACKAGE* to ~s") *package*)
   (throw 'debug 'continue))
 
 ;; Components of the Break-Loop:
@@ -119,6 +121,7 @@ If anything else, printed.")
 
 (defun debug-help () (help) (throw 'debug 'continue))
 (defun debug-unwind () (throw 'debug 'unwind))
+(defun debug-quit () (throw 'debug 'abort-to-top))
 (defun debug-mode-1 () (setq *debug-mode* 1) (throw 'debug 'continue))
 (defun debug-mode-2 () (setq *debug-mode* 2) (throw 'debug 'continue))
 (defun debug-mode-3 () (setq *debug-mode* 3) (throw 'debug 'continue))
@@ -152,12 +155,13 @@ If anything else, printed.")
 (defun get-frame-limit ()
   (let (number)
     (loop
-     (format *debug-io* "
-~%Enter the limit for max. frames to print or ':all' for all: ")
+     (write-string (TEXT "
+Enter the limit for max. frames to print or ':all' for all: ") *debug-io*)
      (setq number (read-from-string (read-line *debug-io* nil nil)))
      (if (or (integerp number) (eq number :all))
        (return)
-       (format *debug-io* "~&~A is not a number. Try again." number)))
+       (format *debug-io* (TEXT "~&~A is not a number. Try again.")
+               number)))
     (unless (eq number :all)
       number)))
 
@@ -170,18 +174,13 @@ If anything else, printed.")
 (defun debug-backtrace (&optional (mode *debug-mode*)
                                   (limit *debug-print-frame-limit*)
                                   (prompt-limit-p nil))
-  (let ((frame (frame-down-1 (frame-up-1 *frame-limit1* mode) mode))
-        (local-limit (or (and prompt-limit-p
-			      (get-frame-limit))   ; local limit takes precedence!
-			 limit)))
-    (do ((i 0 (1+ i)))
-        ((and local-limit (>= i local-limit)) nil)
-      (describe-frame *standard-output* frame)
-      (when (eq frame (setq frame (frame-up-1 frame mode)))
-        (return)))
+  (let ((frame-count
+         (show-stack mode       ; local limit takes precedence:
+                     (or (and prompt-limit-p (get-frame-limit)) limit)
+                     (frame-down-1 (frame-up-1 *frame-limit1* mode) mode))))
+    (format *debug-io* (TEXT "~&Printed ~D frames") frame-count)
     (throw 'debug 'continue)))
 
-
 (defun debug-backtrace-1 () (debug-backtrace 1))
 (defun debug-backtrace-2 () (debug-backtrace 2))
 (defun debug-backtrace-3 () (debug-backtrace 3))
@@ -201,8 +200,7 @@ If anything else, printed.")
   (throw 'debug 'continue))
 
 (defun debug-return ()
-  (return-from-eval-frame *debug-frame*
-                          (read-form (ENGLISH "Values: ")))
+  (return-from-eval-frame *debug-frame* (read-form (TEXT "Values: ")))
   (throw 'debug 'continue))
 (defun debug-continue () (throw 'debug 'quit))
 
@@ -210,69 +208,77 @@ If anything else, printed.")
 (defun debug-print-error ()
   ;; condition is local to break-loop so have to go back there
   (throw 'debug 'print-error))
+(defun debug-inspect-error ()
+  (throw 'debug 'inspect-error))
 
 ;;; print it
 (defun print-error (condition)
-  (format t "~%Recent Error is: >> ~A <<" (print-condition condition nil)))
-
+  (format *debug-io* (TEXT "~%Last error is: >> ~A <<")
+          (print-condition condition nil)))
 
 ;; extended commands
 (defun commands0 ()
   (list
-   (ENGLISH "
+   (TEXT "
 Help (abbreviated :h) = this list
 Use the usual editing capabilities.
-(quit) or (exit) leaves CLISP.")
+\(quit) or (exit) leaves CLISP.")
 
    (cons "Help"         #'debug-help)
    (cons ":h"           #'debug-help)))
 
 (defun commands1 ()
   (list
-   (ENGLISH "
+   (TEXT "
 Commands may be abbreviated as shown in the second column.
-COMMAND        ABBR             DESCRIPTION
-Help           :h (or ?)        this command list
-Error          :e               Print the recent Error Message
-Abort          :a               abort to the next recent input loop
-Unwind         :uw              abort to the next recent input loop
-Reset          :re              toggle *PACKAGE* and *READTABLE* between the
-                                local bindings and the sane values
-Mode-1         :m1              inspect all the stack elements
-Mode-2         :m2              inspect all the frames
-Mode-3         :m3              inspect only lexical frames
-Mode-4         :m4              inspect only EVAL and APPLY frames (default)
-Mode-5         :m5              inspect only APPLY frames
-Where          :w               inspect this frame
-Up             :u               go up one frame, inspect it
-Top            :t               go to top frame, inspect it
-Down           :d               go down one frame, inspect it
-Bottom         :b               go to bottom (most recent) frame, inspect it
-Backtrace-1    :bt1             list all stack elements
-Backtrace-2    :bt2             list all frames
-Backtrace-3    :bt3             list all lexical frames
-Backtrace-4    :bt4             list all EVAL and APPLY frames
-Backtrace-5    :bt5             list all APPLY frames
-Backtrace      :bt              list stack in current mode
-Backtrace-l    :bl              list stack in current mode.
-                                Limit of frames to print will be prompted for.
-Frame-limit    :fl              set the frame-limit. This many frames will
-                                be printed in a backtrace at most.
-Break+         :br+             set breakpoint in EVAL frame
-Break-         :br-             disable breakpoint in EVAL frame
-Redo           :rd              re-evaluate form in EVAL frame
-Return         :rt              leave EVAL frame, prescribing the return values")
+COMMAND        ABBR     DESCRIPTION
+Help           :h, ?    this command list
+Error          :e       Print the last error message
+Inspect        :i       Inspect the last error
+Abort          :a       abort to the next recent input loop
+Unwind         :uw      abort to the next recent input loop
+Reset          :re      toggle *PACKAGE* and *READTABLE* between the
+                          local bindings and the sane values
+Quit           :q       quit to the top-level input loop
+Mode-1         :m1      inspect all the stack elements
+Mode-2         :m2      inspect all the frames
+Mode-3         :m3      inspect only lexical frames
+Mode-4         :m4      inspect only EVAL and APPLY frames (default)
+Mode-5         :m5      inspect only APPLY frames
+Where          :w       inspect this frame
+Up             :u       go up one frame, inspect it
+Top            :t       go to top frame, inspect it
+Down           :d       go down one frame, inspect it
+Bottom         :b       go to bottom (most recent) frame, inspect it
+Backtrace-1    :bt1     list all stack elements
+Backtrace-2    :bt2     list all frames
+Backtrace-3    :bt3     list all lexical frames
+Backtrace-4    :bt4     list all EVAL and APPLY frames
+Backtrace-5    :bt5     list all APPLY frames
+Backtrace      :bt      list stack in current mode
+Backtrace-l    :bl      list stack in current mode.
+                          Limit of frames to print will be prompted for.
+Frame-limit    :fl      set the frame-limit. This many frames will
+                          be printed in a backtrace at most.
+Break+         :br+     set breakpoint in EVAL frame
+Break-         :br-     disable breakpoint in EVAL frame
+Redo           :rd      re-evaluate form in EVAL frame
+Return         :rt      leave EVAL frame, prescribing the return values")
    (cons "Help"         #'debug-help  )
    (cons ":h"           #'debug-help  )
    (cons "?"            #'debug-help  )
    (cons "Error"        #'debug-print-error)
    (cons ":e"           #'debug-print-error)
+   (cons "Inspect"      #'debug-inspect-error)
+   (cons ":i"           #'debug-inspect-error)
    (cons "Abort"        #'debug-unwind)
    (cons ":a"           #'debug-unwind)
    (cons "Unwind"       #'debug-unwind)
    (cons ":uw"          #'debug-unwind)
    (cons "Reset"        #'debug-reset-io)
    (cons ":re"          #'debug-reset-io)
+   (cons "Quit"         #'debug-quit)
+   (cons ":q"           #'debug-quit)
    (cons "Mode-1"       #'debug-mode-1)
    (cons ":m1"          #'debug-mode-1)
    (cons "Mode-2"       #'debug-mode-2)
@@ -323,19 +329,18 @@ Return         :rt              leave EVAL frame, prescribing the return values"
 
 (defun commands3 ()
   (list
-   (ENGLISH "
+   (TEXT "
 Continue       :c       continue evaluation")
    (cons "Continue"     #'debug-continue)
    (cons ":c"           #'debug-continue)))
 
 (defun commands4 ()
   (list
-   (ENGLISH "
-Continue       :c       continue evaluation
+   (TEXT "
 Step           :s       step into form: evaluate this form in single step mode
 Next           :n       step over form: evaluate this form at once
 Over           :o       step over this level: evaluate at once up to the next return
-Continue       :c      switch off single step mode, continue evaluation
+Continue       :c       switch off single step mode, continue evaluation
 -- Step-until :su, Next-until :nu, Over-until :ou, Continue-until :cu --
            same as above, specify a condition when to stop")
    (cons "Step"         #'(lambda () (throw 'stepper 'into)))
@@ -357,23 +362,29 @@ Continue       :c      switch off single step mode, continue evaluation
    (cons "Continue-until" #'(lambda () (throw 'stepper (values 'continue t))))
    (cons ":cu"          #'(lambda () (throw 'stepper (values 'continue t))))))
 
+(defun commands (may-continue commandsr)
+  (nconc (commands1)
+         (when (eval-frame-p *debug-frame*)
+           (commands2))
+         (when may-continue
+           (commands3))
+         commandsr))
+
 ;; Main-Loop with additional help-command
 (defun main-loop ()
   (setq *break-count* 0)
-  (driver                               ; build driver-frame; do #'lambda "infinitely"
+  (driver                 ; build driver-frame; do #'lambda "infinitely"
    #'(lambda ()
-       (catch 'debug                    ; catch the (throw 'debug ...)
-         (if                            ; read-eval-print INPUT-line
-             (read-eval-print
-               (string-concat (prompt-string1)
-                              (prompt-string2)
-                              (prompt-string3))
-               (copy-list (commands0)))
-                                        ; T -> #<EOF>
-           (exit)
-                                        ; NIL -> form is already evaluated
-                                        ; result has been printed
-           )))))
+       (catch 'debug            ; catch the (throw 'debug ...)
+         (when (read-eval-print   ; read-eval-print INPUT-line
+                (string-concat (prompt-string1)
+                               (prompt-string2)
+                               (prompt-string3))
+                (commands0))
+           ;; T -> #<EOF>
+           ;; NIL -> form is already evaluated
+           ;;        result has been printed
+           (exit))))))
 
 (setq *driver* #'main-loop)
 
@@ -411,7 +422,7 @@ Continue       :c      switch off single step mode, continue evaluation
       (if (> *recursive-error-count* 3)
         (progn
           (setq *recursive-error-count* 0)
-          (write-string (ENGLISH "Unprintable error message.")
+          (write-string (TEXT "Unprintable error message.")
                         *error-output*))
         (sys::print-condition condition *error-output*)))
 
@@ -422,48 +433,49 @@ Continue       :c      switch off single step mode, continue evaluation
       (if continuable
         (when interactive-p
           (terpri *debug-io*)
-          (write-string (ENGLISH "You can continue (by typing 'continue').")
+          (write-string (TEXT "You can continue (by typing 'continue').")
                         *debug-io*))
         (progn
           (terpri *debug-io*)
           (when interactive-p
-            (write-string (ENGLISH "If you continue (by typing 'continue'): ")
+            (write-string (TEXT "If you continue (by typing 'continue'): ")
                           *debug-io*))
           (princ may-continue *debug-io*)))))
 
   (when condition
-    (let ((restarts (remove may-continue (compute-restarts condition))))
+    (let ((restarts (remove may-continue (compute-restarts condition)))
+          (restarts-help (if may-continue
+                           (TEXT "The following restarts are also available:")
+                           (TEXT "The following restarts are available:"))))
       (when restarts
         (when interactive-p
           (terpri *debug-io*)
-          (write-string
-            (if may-continue
-              (ENGLISH "The following restarts are available, too:")
-              (ENGLISH "The following restarts are available:"))
-            *debug-io*))
+          (write-string restarts-help *debug-io*))
         (let ((counter 0))
           (dolist (restart restarts)
             (let* ((command
-                    (string-concat "R" (sys::decimal-string (incf counter))))
-                   (helpstring (string-concat "
-" command " = " (princ-to-string restart))))
+                    (string-concat ":R" (sys::decimal-string (incf counter))))
+                   (name (string (restart-name restart)))
+                   (helpstring (format nil "~%~A~15T~A~24T~A" name command
+                                       (princ-to-string restart)))
+                   (restart restart)  ; for FUNC
+                   (func #'(lambda () (invoke-restart-interactively restart))))
               ;; display the restarts:
               (when interactive-p
                 (write-string helpstring *debug-io*))
               (push helpstring commandsr)
-              ;; put it into the  commandsr list.
-              (push
-                (cons command
-                      (let ((restart restart))
-                        #'(lambda ()
-                            (invoke-restart-interactively restart))))
-                commandsr)))
+              ;; put it into the commandsr list.
+              (push (cons command func) commandsr)
+              (push (cons name func) commandsr)))
           (terpri *debug-io*)
-          (setq commandsr (nreverse commandsr))))))
+          (setq commandsr (cons (string-concat (string #\Newline) restarts-help)
+                                (nreverse commandsr)))))))
   (force-output *debug-io*)
 
   (tagbody
-    (clear-input *debug-io*)    ; because the user didn't expect a break loop
+    (makunbound '*terminal-read-stream*)
+    (makunbound '*terminal-read-open-object*)
+    (clear-input *debug-io*) ; because the user did not expect a break loop
     (let* ((*break-count* (1+ *break-count*))
            (stream (make-synonym-stream '*debug-io*))
            (*standard-input* stream)
@@ -479,38 +491,34 @@ Continue       :c      switch off single step mode, continue evaluation
            (*frame-limit2* (frame-limit2))
            (*debug-mode* 4)
            (*debug-frame*
-             (frame-down-1 (frame-up-1 *frame-limit1* *debug-mode*) *debug-mode*)))
+            (frame-down-1 (frame-up-1 *frame-limit1* *debug-mode*)
+                          *debug-mode*))
+           (commands-list (commands may-continue commandsr)))
       (driver
-        ;; build driver frame and repeat #'lambda (infinitely; ...)
-        #'(lambda ()
-            (case
-                (catch 'debug             ; catch (throw 'debug ...) and analyse
-
-                  ;; build environment *debug-frame* that is valid/equal for/to *debug-frame*
-                  (same-env-as *debug-frame*
-                    #'(lambda ()
-                        (if    ; read-eval-print INPUT-line
-                            (read-eval-print
-                              prompt
-                              (nconc (copy-list (commands1))
-                                     (when (eval-frame-p *debug-frame*)
-                                       (copy-list (commands2)))
-                                     (when may-continue
-                                       (copy-list (commands3)))
-                                     commandsr))
-                                        ; T -> #<EOF>
-                          (throw 'debug (if may-continue 'quit 'unwind))
-                          ;; NIL -> form is already evaluated; result has been printed
-                          ))))
-              (print-error (print-error condition)) ; print the recent error-message
-              (unwind (go unwind))
-              (quit                       ; reached only, if may-continue is T
-                (if continuable
-                  (go quit)
-                  (invoke-restart-interactively may-continue)))
-              (t )                        ; other cases, especially continue
-              ))))
-    unwind (unwind-to-driver)
+       ;; build driver frame and repeat #'lambda (infinitely; ...)
+       #'(lambda ()
+           (case (catch 'debug    ; catch (throw 'debug ...) and analyse
+                   ;; build environment *debug-frame*
+                   ;; which is valid/equal for/to *debug-frame*
+                   (same-env-as *debug-frame*
+                     #'(lambda ()
+                         (if (read-eval-print ; read-eval-print INPUT-line
+                              prompt commands-list)
+                             ;; T -> #<EOF>
+                             ;; NIL -> form is already evaluated;
+                             ;;        result has been printed
+                             (throw 'debug (if may-continue 'quit 'unwind))))))
+             ((print-error) (print-error condition))
+             ((inspect-error) (inspect condition))
+             ((unwind) (go unwind))
+             ((abort-to-top) (go abort-to-top))
+             ((quit)            ; reached only if may-continue is T
+              (if continuable
+                (go quit)
+                (invoke-restart-interactively may-continue)))
+             (t )))))           ; other cases, especially continue
+    unwind (unwind-to-driver nil)
+    abort-to-top (unwind-to-driver t)
     quit))
 
 (setq *break-driver* #'break-loop)
@@ -526,7 +534,7 @@ Continue       :c      switch off single step mode, continue evaluation
 (defvar *step-watch* nil)		; terminating condition
 
 (defmacro step (form)
-  "(STEP form), CLTL S. 441"
+  "(STEP form), CLTL p. 441"
   `(let* ((*step-level* 0)
           (*step-quit* most-positive-fixnum)
           (*step-watch* nil)
@@ -536,15 +544,15 @@ Continue       :c      switch off single step mode, continue evaluation
 (defun step-values (values)
   (let ((*standard-output* *debug-io*))
     (terpri #|*debug-io*|#)
-    (write-string (ENGLISH "step ") #|*debug-io*|#)
+    (write-string (TEXT "step ") #|*debug-io*|#)
     (write *step-level* #|:stream *debug-io*|#)
     (write-string " ==> " #|*debug-io*|#)
     (case (length values)
-      (0 (write-string (ENGLISH "no values") #|*debug-io*|#))
-      (1 (write-string (ENGLISH "value: ") #|*debug-io*|#)
+      (0 (write-string (TEXT "no values") #|*debug-io*|#))
+      (1 (write-string (TEXT "value: ") #|*debug-io*|#)
          (write (car values) #|:stream *debug-io*|#))
       (t (write (length values) #|:stream *debug-io*|#)
-         (write-string (ENGLISH " values: ") #|*debug-io*|#)
+         (write-string (TEXT " values: ") #|*debug-io*|#)
          (do ((L values))
              ((endp L))
            (write (pop L) #|:stream *debug-io*|#)
@@ -553,12 +561,11 @@ Continue       :c      switch off single step mode, continue evaluation
 
 (defun step-hook-fn (form &optional (env *toplevel-environment*))
   (let ((*step-level* (1+ *step-level*)))
-    (when (>= *step-level* *step-quit*) ; as long as *step-level* >= *step-quit*
+    (when (>= *step-level* *step-quit*) ; while *step-level* >= *step-quit*
       (if (and *step-watch* (funcall *step-watch*)) ; and no Breakpoint,
         (setq *step-quit* most-positive-fixnum)
         (return-from step-hook-fn	; the Stepper remains passive
-          (evalhook form nil nil env)	; (e.g. it simply evaluates the Form)
-	  )))
+          (evalhook form nil nil env)))) ; (e.g. it simply evaluates the Form)
     (tagbody
       (let* ((stream (make-synonym-stream '*debug-io*))
              (*standard-input* stream)
@@ -573,9 +580,12 @@ Continue       :c      switch off single step mode, continue evaluation
              (*frame-limit1* (frame-limit1 11))
              (*frame-limit2* (frame-limit2))
              (*debug-mode* 4)
-             (*debug-frame* (frame-down-1 (frame-up-1 *frame-limit1* *debug-mode*) *debug-mode*)))
+             (*debug-frame*
+              (frame-down-1 (frame-up-1 *frame-limit1* *debug-mode*)
+                            *debug-mode*))
+             (commands-list (commands nil (commands4))))
         (fresh-line #|*debug-io*|#)
-        (write-string (ENGLISH "step ") #|*debug-io*|#)
+        (write-string (TEXT "step ") #|*debug-io*|#)
         (write *step-level* #|:stream *debug-io*|#)
         (write-string " --> " #|*debug-io*|#)
         (write form #|:stream *debug-io*|# :length 4 :level 3)
@@ -593,50 +603,71 @@ Continue       :c      switch off single step mode, continue evaluation
                               ;; build environment *debug-frame* that
                               ;; is valid/equal for/to *debug-frame*
                               #'(lambda ()
-                                  (if ; get/read INPUT-line
-                                      (read-eval-print
-                                        prompt
-                                        (nconc (copy-list (commands1))
-                                               (when (eval-frame-p *debug-frame*)
-                                                 (copy-list (commands2)))
-                                               (copy-list (commands4))
-                                               ))
-					; T -> #<EOF>
-                                    (go continue)
-					; NIL -> form is already evaluated; result has been printed
+                                  (if (read-eval-print ; get/read INPUT-line
+                                       prompt commands-list)
+                                      ;; T -> #<EOF>
+                                      (go continue)
+                                      ;; NIL -> form is already evaluated;
+                                      ;;        result has been printed
                                     #|(throw 'debug 'continue)|#
-                                    ))))
-                        (unwind (go unwind))
-                        (t )		; other cases, especially continue
-                        ))))
+                                      ))))
+                        ((unwind) (go unwind))
+                        ((abort-to-top) (go abort-to-top))
+                        (t ))))) ; other cases, especially continue
             (when watchp
-              (let ((form (read-form (ENGLISH "condition when to stop: "))))
+              (let ((form (read-form (TEXT "condition when to stop: "))))
                 (setq *step-watch*
-                        ;; Funktion, that evaluates 'form' in/with *debug-frame*
-                        (eval-at *debug-frame*
-                                 `(function (lambda () ,form))))))
+                      ;; function which evaluates 'form' in/with *debug-frame*
+                      (eval-at *debug-frame* `(function (lambda () ,form))))))
             (case what
               (into (go into))
               (over (go over))
               (over-this-level (go over-this-level))
               (continue (go continue))))))
-      unwind
-        (unwind-to-driver)
-      into
-        (return-from step-hook-fn
-          (step-values
-            (multiple-value-list (evalhook form #'step-hook-fn nil env))))
-      over-this-level
-        (setq *step-quit* *step-level*)	; keep the Stepper sleeping
-      over
-        (return-from step-hook-fn
-          (step-values
-            (multiple-value-list (evalhook form nil nil env))))
-      continue
-        (setq *step-quit* 0)
-        (go over))))
+     unwind (unwind-to-driver nil)
+     abort-to-top (unwind-to-driver t)
+     into
+      (return-from step-hook-fn
+        (step-values
+         (multiple-value-list (evalhook form #'step-hook-fn nil env))))
+     over-this-level
+      (setq *step-quit* *step-level*)	; keep the Stepper sleeping
+     over
+      (return-from step-hook-fn
+        (step-values
+         (multiple-value-list (evalhook form nil nil env))))
+     continue
+      (setq *step-quit* 0)
+      (go over))))
 
 ;;;--------------------------------------------------------------------------
+#+unix
+(defun make-xterm-io-stream ()
+  (let* ((pipe (with-open-stream (s (make-pipe-input-stream
+                                     "mktemp /tmp/clisp-x-io-XXXXXX"))
+                 (read-line s)))
+         (title "CLISP I/O")
+         tty-name xio
+         (clos::*warn-if-gf-already-called* nil))
+    (shell (format nil "rm -f ~a; mknod ~a p; ~
+                   xterm -n ~s -T ~s -e 'tty >> ~a; cat ~a' &"
+                   pipe pipe title title pipe pipe))
+    (setq tty-name
+          (with-open-file (s pipe :direction :input) (read-line s))
+          xio (make-two-way-stream
+               (open tty-name :direction :input)
+               (open tty-name :direction :output)))
+    (defmethod close :after ((x (eql xio)) &rest junk)
+      (declare (ignore x junk))
+      (with-open-file (s pipe :direction :output)
+        (write-line (TEXT "Bye.") s))
+      (delete-file pipe)
+      (close (two-way-stream-input-stream xio))
+      (close (two-way-stream-output-stream xio))
+      (let ((clos::*warn-if-gf-already-called* nil))
+        (remove-method #'close (find-method #'close '(:after) `((eql ,xio))))))
+    xio))
+;;;--------------------------------------------------------------------------
 
 ;; Now that condition.lisp is loaded and *break-driver* has a value:
 ;; Activate the Condition System.
